/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.objects.te.TEPitKiln;
import net.dries007.tfc.util.Helpers;

public class BlockPitKiln extends Block implements ITileEntityProvider
{
    public static final PropertyBool FULL = PropertyBool.func_177716_a("full");
    public static final PropertyBool LIT = PropertyBool.func_177716_a("lit");
    protected static final AxisAlignedBB AABB = new AxisAlignedBB(0, 0, 0, 1, 1D / 16D, 1);

    public BlockPitKiln()
    {
        super(Material.field_151594_q);
        func_149711_c(0.5f);
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(FULL, false).func_177226_a(LIT, false));
        TileEntity.func_190560_a(TEPitKiln.ID.toString(), TEPitKiln.class);
    }

    @Nullable
    @Override
    public TileEntity func_149915_a(World worldIn, int meta)
    {
        return new TEPitKiln();
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_185481_k(IBlockState state)
    {
        // This is required for the fire code, because forge doesn't 'fix' it to use the location sensitive version.
        return true;
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P();
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return 0;
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te == null) return state;
        return state.func_177226_a(BlockPitKiln.LIT, te.isLit()).func_177226_a(BlockPitKiln.FULL, te.hasFuel());
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return true;
    }

    @Override
    @SuppressWarnings("deprecation")
    public EnumBlockRenderType func_149645_b(IBlockState state)
    {
        return EnumBlockRenderType.ENTITYBLOCK_ANIMATED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        // todo: depend on fill level?
        return state.func_185899_b(source, pos).func_177229_b(FULL) ? field_185505_j : AABB;
    }

    @Override
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
        if (!worldIn.isSideSolid(pos.func_177982_a(0, -1, 0), EnumFacing.UP))
            worldIn.func_175655_b(pos, true);
        if (blockIn == Blocks.field_150480_ab)
        {
            TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
            if (te != null) te.tryLight();
        }
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te != null) te.onBreakBlock();
        super.func_180663_b(worldIn, pos, state); // todo: drop items
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te == null) return true;
        te.onRightClick(playerIn, playerIn.func_184586_b(hand), hitX < 0.5, hitZ < 0.5);
        return true;
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, FULL, LIT);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean isSideSolid(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        return state.func_185899_b(world, pos).func_177229_b(FULL);
    }

    @Override
    public boolean isBurning(IBlockAccess world, BlockPos pos)
    {
        return world.func_180495_p(pos).func_185899_b(world, pos).func_177229_b(LIT);
    }

    @Override
    public int getFlammability(IBlockAccess world, BlockPos pos, EnumFacing face)
    {
        return 0;
    }

    @Override
    public int getFireSpreadSpeed(IBlockAccess world, BlockPos pos, EnumFacing face)
    {
        return world.func_180495_p(pos).func_185899_b(world, pos).func_177229_b(LIT) ? 120 : 0; // Twice as much as the highest vanilla level (60)
    }

    @Override
    public boolean isFireSource(World world, BlockPos pos, EnumFacing side)
    {
        return world.func_180495_p(pos).func_185899_b(world, pos).func_177229_b(LIT);
    }
}
