/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.stone;

import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.BlockFalling;
import net.minecraft.block.SoundType;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.EnumPlantType;
import net.minecraftforge.common.IPlantable;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.Rock;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.entity.EntityFallingBlockTFC;
import net.dries007.tfc.objects.items.rock.ItemRock;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.IFallingBlock;
import net.dries007.tfc.util.InsertOnlyEnumTable;
import net.dries007.tfc.util.OreDictionaryHelper;

public class BlockRockVariant extends Block implements IFallingBlock
{
    private static final InsertOnlyEnumTable<Rock, Rock.Type, BlockRockVariant> TABLE = new InsertOnlyEnumTable<>(Rock.class, Rock.Type.class);

    public static BlockRockVariant get(Rock rock, Rock.Type type)
    {
        return TABLE.get(rock, type);
    }

    public final Rock.Type type;
    public final Rock rock;

    public BlockRockVariant(Rock.Type type, Rock rock)
    {
        super(type.material);
        TABLE.put(rock, type, this);
        this.type = type;
        this.rock = rock;
        if (type.isGrass) func_149675_a(true);
        switch (type)
        {
            case BRICKS:
            case RAW:
                func_149672_a(SoundType.field_185851_d);
                func_149711_c(2.0F).func_149752_b(10.0F);
                setHarvestLevel("pickaxe", 0);
                break;
            case COBBLE:
            case SMOOTH:
                func_149672_a(SoundType.field_185851_d);
                func_149711_c(1.5F).func_149752_b(10.0F);
                setHarvestLevel("pickaxe", 0);
                break;
            case SAND:
                func_149672_a(SoundType.field_185855_h);
                func_149711_c(0.5F);
                setHarvestLevel("shovel", 0);
                break;
            case DIRT:
                func_149672_a(SoundType.field_185849_b);
                func_149711_c(0.5F);
                setHarvestLevel("shovel", 0);
                break;
            case GRAVEL:
                func_149672_a(SoundType.field_185849_b);
                func_149711_c(0.6F);
                setHarvestLevel("shovel", 0);
                break;
            case CLAY:
            case CLAY_GRASS:
            case GRASS:
            case DRY_GRASS:
                func_149672_a(SoundType.field_185850_c);
                func_149711_c(0.6F);
                setHarvestLevel("shovel", 0);
                break;
        }
        OreDictionaryHelper.registerRockType(this, type, rock);
    }

    public BlockRockVariant getVariant(Rock.Type t)
    {
        return TABLE.get(rock, t);
    }

    @Override
    public boolean shouldFall(IBlockState state, World world, BlockPos pos)
    {
        return type.isAffectedByGravity && IFallingBlock.super.shouldFall(state, world, pos);
    }

    @Override
    public void func_180645_a(World world, BlockPos pos, IBlockState state, Random rand)
    {
        if (world.field_72995_K) return;
        if (type.isGrass) Helpers.spreadGrass(world, pos, state, rand);
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        super.func_180650_b(worldIn, pos, state, rand);
        if (worldIn.field_72995_K) return;
        if (shouldFall(state, worldIn, pos))
        {
            if (!BlockFalling.field_149832_M && worldIn.func_175707_a(pos.func_177982_a(-32, -32, -32), pos.func_177982_a(32, 32, 32)))
            {
                worldIn.func_72838_d(new EntityFallingBlockTFC(worldIn, pos, this, worldIn.func_180495_p(pos)));
            }
            else
            {
                worldIn.func_175698_g(pos);
                pos = pos.func_177982_a(0, -1, 0);
                while (canFallThrough(worldIn.func_180495_p(pos)) && pos.func_177956_o() > 0)
                    pos = pos.func_177982_a(0, -1, 0);
                if (pos.func_177956_o() > 0) worldIn.func_175656_a(pos.func_177984_a(), state); // Includes Forge's fix for data loss.
            }
        }
    }

    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState stateIn, World worldIn, BlockPos pos, Random rand)
    {
        if (!this.type.isAffectedByGravity) return;
        if (rand.nextInt(16) != 0) return;
        if (shouldFall(stateIn, worldIn, pos))
        {
            double d0 = (double) ((float) pos.func_177958_n() + rand.nextFloat());
            double d1 = (double) pos.func_177956_o() - 0.05D;
            double d2 = (double) ((float) pos.func_177952_p() + rand.nextFloat());
            worldIn.func_175688_a(EnumParticleTypes.FALLING_DUST, d0, d1, d2, 0.0D, 0.0D, 0.0D, Block.func_176210_f(stateIn));
        }
    }

    // IDK what the alternative is supposed to be, so I'm gonna continue using this.
    @SuppressWarnings("deprecation")
    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
        if (shouldFall(state, worldIn, pos)) worldIn.func_175684_a(pos, this, func_149738_a(worldIn));
    }

    @Override
    public int func_149738_a(World worldIn)
    {
        return 1; // todo: tickrate in vanilla is 2, in tfc1710 it's 10
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        if (shouldFall(state, worldIn, pos)) worldIn.func_175684_a(pos, this, func_149738_a(worldIn));
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        switch (type)
        {
            case RAW:
                return ItemRock.get(rock);
            case CLAY:
            case CLAY_GRASS:
                return Items.field_151119_aD; // todo: own clay or event for clay making?
            default:
                return super.func_180660_a(state, rand, fortune);
            case GRASS:
            case DRY_GRASS:
                return Item.func_150898_a(get(rock, Rock.Type.DIRT));
        }
    }

    @Override
    public int func_180651_a(IBlockState state)
    {
        return func_176201_c(state);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k()
    {
        return type.isGrass ? BlockRenderLayer.CUTOUT : BlockRenderLayer.SOLID;
    }

    @Override
    public int quantityDropped(IBlockState state, int fortune, Random random)
    {
        // todo: see how 1710 handles this
        switch (type)
        {
            case CLAY:
            case CLAY_GRASS:
                return 4;
            default:
                return super.quantityDropped(state, fortune, random);
        }
    }

    @Override
    public boolean canSustainPlant(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing direction, IPlantable plantable)
    {
        EnumPlantType plantType = plantable.getPlantType(world, pos.func_177972_a(direction));

        switch (plantType)
        {
            case Plains:
                return type == Rock.Type.DIRT || type == Rock.Type.GRASS; // todo: dry grass?
            case Crop:
                return type == Rock.Type.DIRT || type == Rock.Type.GRASS; // todo: dry grass? Should this even be true? Might be required for wild crops.
            case Desert:
                return type == Rock.Type.SAND;
            case Cave:
                return true;
            case Water:
                return false;
            case Beach:
                return (type == Rock.Type.DIRT || type == Rock.Type.GRASS || type == Rock.Type.SAND || type == Rock.Type.DRY_GRASS) && // todo: dry grass?
                        (BlocksTFC.isWater(world.func_180495_p(pos.func_177982_a(1, 0, 0))) ||
                                BlocksTFC.isWater(world.func_180495_p(pos.func_177982_a(-1, 0, 0))) ||
                                BlocksTFC.isWater(world.func_180495_p(pos.func_177982_a(0, 0, 1))) ||
                                BlocksTFC.isWater(world.func_180495_p(pos.func_177982_a(0, 0, -1))));
            case Nether:
                return false;
        }

        return false;
    }
}
